package com.nisum.impl;

import com.genericAvro.PersonData;
import com.nisum.constants.IKafkaConstants;
import com.nisum.model.Person;
import io.confluent.kafka.serializers.KafkaAvroSerializer;
import org.apache.kafka.clients.producer.*;
import org.apache.kafka.common.serialization.StringSerializer;
import org.apache.spark.sql.ForeachWriter;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.Properties;

public class ForEachWriterImpl extends ForeachWriter<Person> {

    private static final Logger LOGGER = LoggerFactory.getLogger(ForEachWriterImpl.class);

    private Properties properties = new Properties();
    private Producer<String, PersonData> producer;
    private String topic;
    private PersonData personData;
    private ProducerRecord<String, PersonData> producerRecord;

    @Override
    public boolean open ( long l, long l1 ) {
        LOGGER.info("ForEach Writer for every stream started :: open");
        properties.setProperty("bootstrap.servers", IKafkaConstants.KAFKA_SERVER);
        properties.setProperty("acks", "all");
        properties.setProperty("retries", "1");
        properties.setProperty("key.serializer", StringSerializer.class.getName());
        properties.setProperty("value.serializer", KafkaAvroSerializer.class.getName());
        properties.setProperty("schema.registry.url", IKafkaConstants.SCHEMA_REGISTRY_URL);
        System.out.println("Closed open");
        return true;
    }

    @Override
    public void process ( Person person ) {
        try {
            LOGGER.info("ForEach Writer Process for every stream started :: Process");
            producer = new KafkaProducer<String, PersonData>(properties);
            topic = person.getName().trim().replaceAll(" ", "");
            personData = mapPerson(person);
            System.out.println("Inside process");
            producerRecord = new ProducerRecord<String, PersonData>("topic", personData);
            producer.send(producerRecord, new Callback() {
                @Override
                public void onCompletion ( RecordMetadata metadata, Exception exception ) {
                    if (exception == null) {
                        System.out.println("metadata : " + metadata);
                    } else {
                        LOGGER.info("Exception in ForEach Writer :: {}",exception.getMessage());
                    }
                }
            });

        } catch ( Exception e ) {
            LOGGER.info("Exception in ForEach Writer :: {}",e.getMessage());

        }

    }

    private PersonData mapPerson ( Person person ) {
        return PersonData.newBuilder()
                .setName(person.getName())
                .setAge(person.getAge()).build();
    }

    @Override
    public void close ( Throwable throwable ) {
        System.out.println("Inside close");
        System.out.println("Offer Processed ");
    }


}
