package com.nisum.job;

import com.nisum.config.SparkSessionConfig;
import com.nisum.constants.IKafkaConstants;
import com.nisum.impl.ForEachWriterImpl;
import org.apache.spark.sql.Dataset;
import org.apache.spark.sql.Row;
import org.apache.spark.sql.SparkSession;
import org.apache.spark.sql.streaming.OutputMode;
import org.apache.spark.sql.streaming.StreamingQueryException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class KafkaDataReaderJob {

    private static final SparkSession sparkSession = SparkSessionConfig.sparkSession();

    private static final Logger LOGGER = LoggerFactory.getLogger(KafkaDataReaderJob.class);

    public void read () {
        LOGGER.info("Kafka Data reader job got started");
        Dataset<Row> rowDataSet = sparkSession
                .readStream()
                .format("kafka")
                .option("kafka.bootstrap.servers", IKafkaConstants.KAFKA_SERVER)
                .option("subscribe", IKafkaConstants.TOPIC_NAME)
                .option("startingOffsets", IKafkaConstants.START_OFFSET)
                .option("failOnDataLoss", "false")
                .load()
                .selectExpr("CAST(key AS STRING)", "CAST(value AS STRING)");
        rowDataSet.printSchema();

        try {
            rowDataSet.writeStream()
                    .format("console")
                    .option("checkpointLocation", "/tmp/SparkSession")
                    .outputMode(OutputMode.Append())
                    .start()
                    .awaitTermination();
        } catch ( StreamingQueryException e ) {
            LOGGER.error("Exception in KafkaData Reader Job :: {}", e.message());
        } finally {
            sparkSession.close();
        }

    }
}
