import com.genericAvro.PersonData;
import com.nisum.model.Person;
import io.confluent.kafka.serializers.KafkaAvroSerializer;
import org.apache.kafka.clients.producer.KafkaProducer;
import org.apache.kafka.clients.producer.Producer;
import org.apache.kafka.clients.producer.ProducerRecord;
import org.apache.kafka.common.serialization.StringSerializer;
import org.junit.jupiter.api.Test;

import java.util.Arrays;
import java.util.List;
import java.util.Properties;
import java.util.Random;
import java.util.stream.Collectors;
import java.util.stream.IntStream;

public class TestDataGenerator {
    static Properties properties = new Properties();

    @Test
    public void prepareData() {
        generatorTestData();
    }

    /*
     *  Generate Offer Data to offerLkpConnect topic and returns the storeIDs from the offer data for further assertions
     * */
    public List<String> generatorTestData() {

        properties.setProperty("bootstrap.servers", "http://127.0.0.1:9092");
        properties.setProperty("acks", "all");
        properties.setProperty("retries", "10");
        properties.setProperty("key.serializer", StringSerializer.class.getName());
        properties.setProperty("value.serializer", KafkaAvroSerializer.class.getName());
        properties.setProperty("schema.registry.url","http://127.0.0.1:8081" );

        Producer<String, PersonData> producer = new KafkaProducer<>(properties);

        return generatePerson(3).stream().map(personData -> {
            produceMessageToTopic(producer, personData);
            return personData.getName().trim().replaceAll(" ", "");
        }).collect(Collectors.toList());
    }

    private List<PersonData> generatePerson ( int limit ) {
        return IntStream.range(0, limit).mapToObj(i -> {
            Person person = new Person();
            Random random = new Random();

            person.setAge(String.valueOf(random.nextInt(40)));
            person.setName("user"+random.nextInt());
            return mapPerson(person);
        }).collect(Collectors.toList());

    }

    private PersonData mapPerson ( Person person ) {
        return new PersonData(person.getName(),person.getAge());
    }

    private void produceMessageToTopic(Producer<String, PersonData> producer, PersonData personData) {
        ProducerRecord<String, PersonData> producerRecord = new ProducerRecord<>(
                "Nisum", personData

        );
        producer.send(producerRecord, (metadata, exception) -> {
            System.out.println(metadata);
            if (exception == null) {
                System.out.println("msg send to the topic");
                System.out.println(metadata);
            } else {
                exception.printStackTrace();
            }
        });
    }
}
