package com.example.SparkStructedStreaming.job;

import com.example.SparkStructedStreaming.Person;
import com.example.SparkStructedStreaming.Service.ForEachWriterImpl;
import io.confluent.kafka.schemaregistry.client.CachedSchemaRegistryClient;
import io.confluent.kafka.schemaregistry.client.SchemaMetadata;
import io.confluent.kafka.schemaregistry.client.rest.exceptions.RestClientException;
import io.confluent.kafka.serializers.KafkaAvroDeserializer;
import org.apache.avro.Schema;
import org.apache.avro.generic.GenericRecord;
import org.apache.spark.sql.Dataset;
import org.apache.spark.sql.RowFactory;
import org.apache.spark.sql.SparkSession;
import org.apache.spark.sql.avro.SchemaConverters;
import org.apache.spark.sql.streaming.OutputMode;
import org.apache.spark.sql.types.StructType;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.IOException;

import static org.apache.spark.sql.Encoders.bean;

@Service
public class KafkaDataWriterJob {

    @Autowired
    SparkSession sparkSession;

  /* private SparkSession sparkSession = SparkSessionConfig.sparkSession();

    private static final SQLContext sqlContext = SparkSessionConfig.sparkSession().sqlContext();
*/
    public void run() throws IOException, RestClientException {
        CachedSchemaRegistryClient cachedSchemaRegistryClient = new CachedSchemaRegistryClient("http://127.0.0.1:8081", 128);
        SchemaMetadata metadata = cachedSchemaRegistryClient.getLatestSchemaMetadata("Person" + "-value");
        Schema schema = cachedSchemaRegistryClient.getById(metadata.getId());
        StructType dataType = (StructType) SchemaConverters.toSqlType(schema).dataType();

        sparkSession.udf().register("deserialize", (byte[] data) -> {
            CachedSchemaRegistryClient schemaRegistryClient = new CachedSchemaRegistryClient("http://127.0.0.1:8081", 128);
            KafkaAvroDeserializer kafkaAvroDeserializer = new KafkaAvroDeserializer(schemaRegistryClient);
            GenericRecord record = (GenericRecord) kafkaAvroDeserializer.deserialize("", data);

            return RowFactory.create(record.get("name").toString(), record.get("age").toString()); }, dataType);

        Dataset<Person> rowDataSet = sparkSession
                .readStream()
                .format("kafka")
                .option("kafka.bootstrap.servers","http://127.0.0.1:9092")
                .option("subscribe","Person")
                .option("startingOffsets", "earliest")
                .option("failOnDataLoss","false")
                .load()
                .selectExpr("deserialize(value) as value")
                .select("value.*")
                .as(bean(Person.class));

        rowDataSet.printSchema();
        rowDataSet.createOrReplaceTempView("personData");

        try {
            Dataset<Person> personDataSet = sparkSession.sqlContext().sql("select * from personData").as(bean(Person.class));
            personDataSet.writeStream()
                    .foreach(new ForEachWriterImpl())
                    .outputMode(OutputMode.Append())
                    .option("checkpointLocation", "/tmp/OfferLookupKafkaCheckpointagain")
                    .start()
                    .awaitTermination();


        } catch ( Exception e ) {
            e.printStackTrace();
        }
        finally {
            sparkSession.close();
        }

    }

}




   /*         ro.writeStream()
                    .foreach(new ForEachWriterImpl())
                    .outputMode(OutputMode.Append())
                    .start();*/

             /*   rowDataSet.writeStream()
                        .option("spark.mongodb.output.uri", "mongodb://127.0.0.1/test")
                        .outputMode(OutputMode.Append())
                        .option("checkpointLocation", "/tmp/SparkSessionMongo")
                        .foreach(new ForEachWriterImpl())
                        .start()
                        .awaitTermination();;*/

/*
            rowDataSet.writeStream()
                    .option("kafka.bootstrap.servers", "http://127.0.0.1:9092")
                    .outputMode(OutputMode.Append())
                    .option("checkpointLocation", "/tmp/SparkSession")
                    .foreach(new ForEachWriterImpl())
                    .start();*/
